/*
 * linux/arch/mips/vr41xx/nec_osprey.c
 *
 * Platform support for NEC Osprey.
 *
 * Copyright (C) 2000 Bradley D. LaRonde
 * Copyright (C) 2000 Michael Klar
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 */

#include <linux/kernel.h>
#include <linux/delay.h>
#include <asm/io.h>
#include <asm/vr41xx.h>

#define CONTROL_PORT 0x03ffff00
#define LBLC_PHYSADR_START (CONTROL_PORT + VR41XX_ISA_IO)
#define LBLC_PHYSADR_STOP LBLC_PHYSADR_START


static void control_port_setup() {
	// Set up programmable chip select 0 for the control port.
	*VR41XX_PCS0STRA = LBLC_PHYSADR_START & 0xffff;
	*VR41XX_PCS0STPA = LBLC_PHYSADR_STOP & 0xffff;
	*VR41XX_PCS0HIA =  (LBLC_PHYSADR_START >> 16) & 0x0fff;

	// Set up programmable chip select mode.
	*VR41XX_PCSMODE = 0x2;
}

static unsigned char control_port;

static void set_control_port(unsigned char and, unsigned char or) {
	control_port &= and;
	control_port |= or;
	outb(control_port, CONTROL_PORT);
	// printk("set_control_port and %x or %x out %x\n", and, or, control_port);
}

static void display_setup(void)
{
	// Use GP12-15 for color LCD FPD[4-7] output.
	*VR41XX_GPMD1REG |= 0xff00;

	// Set gclk to Hpck ratio.
	*VR41XX_LCDCFGREG1 = 0x0707;

#define VCC_FRAME    0x0
#define IFSIG_FRAME  0x3
#define BIAS_FRAME   0x7

	// Set LCD power control.
	*VR41XX_PWRCONREG1 = BIAS_FRAME;
	*VR41XX_PWRCONREG2 = (0 << 15) | (1 << 14) | (1 << 11) | (0 << 10)
		| (IFSIG_FRAME << 5) | VCC_FRAME;
}

int display_power(int on)
{
	if(on) {
		// Turn on the power.
		*VR41XX_PWRCONREG2 |= (1 << 10);
		mdelay(100);
		set_control_port(~0, 1 << 4);
		display_set_backlight(1);
	}
	else {
		// Turn off the power.
		display_set_backlight(0);
		set_control_port(~(1 << 4), 0);
		mdelay(100);
		*VR41XX_PWRCONREG2 &= ~(1 << 10);
	}
	return 0;
}

int display_get_backlight()
{
	return (control_port & (1 << 5)) ? 1 : 0;
}

int display_set_backlight(int n)
{
	if(n == 0)
		// Turn the backlight off.
		set_control_port(~(1 << 5), 0);
	else
		// Turn the backlight on.
		set_control_port(~0, 1 << 5);
	return 0;
}

int display_get_contrast()
{
	return(control_port & 0x7);
}

int display_set_contrast(int n)
{
	set_control_port(0xf8, n & 0x7);
	return 0;
}

void platdep_setup(void)
{
	control_port_setup();
	set_control_port(0, 0);
	display_setup();
	display_set_contrast(6);
}
