/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * SNI specific PCI support for NEC VR4122.
 *
 * Copyright (C) 2000 Mike McDonald
 */
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/types.h>
#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/vr4122.h>

#ifdef CONFIG_PCI

#define mkaddr(dev, reg)				\
do {							\
    if (dev->bus->number == 0) {			\
	       if (PCI_SLOT(dev->devfn) < 11 ||		\
		   PCI_SLOT(dev->devfn) >= 32 ||	\
		   PCI_FUNC(dev->devfn) >= 8 ||		\
		   reg >= 256)				\
		   return 0xFFFFFFFF;			\
	*(volatile u32 *)VR41XX_PCICONFAREG =		\
            (reg & 0xFC) |				\
	    ( PCI_FUNC(dev->devfn) << 8) |		\
	    (1 << PCI_SLOT(dev->devfn));		\
    } else {						\
	       if (dev->bus->number >= 256 ||		\
		   PCI_SLOT(dev->devfn) >= 32 ||	\
		   PCI_FUNC(dev->devfn) >=8 ||		\
		   reg >= 256)				\
		   return 0xFFFFFFFF;			\
	*(volatile u32 *)VR41XX_PCICONFAREG = 1 |	\
	    (reg & 0xFC) |				\
	    (dev->devfn << 8) |				\
	    (dev->bus->number << 16);			\
    }							\
} while (0);

/*
 * We can't address 8 and 16 bit words directly.  Instead we have to
 * read/write a 32bit word and mask/modify the data we actually want.
 */
static int pcimt_read_config_byte (struct pci_dev *dev,
                                   int where, unsigned char *val)
{
	u32 res;

	mkaddr(dev, where);
	res = *(volatile u32 *)VR41XX_PCICONFDREG;
	res = (le32_to_cpu(res) >> ((where & 3) << 3)) & 0xff;
	*val = res;

	return PCIBIOS_SUCCESSFUL;
}

static int pcimt_read_config_word (struct pci_dev *dev,
                                   int where, unsigned short *val)
{
	u32 res;

	if (where & 1)
		return PCIBIOS_BAD_REGISTER_NUMBER;
	mkaddr(dev, where);
	res = *(volatile u32 *)VR41XX_PCICONFDREG;
	res = (le32_to_cpu(res) >> ((where & 3) << 3)) & 0xffff;
	*val = res;

	return PCIBIOS_SUCCESSFUL;
}

static int pcimt_read_config_dword (struct pci_dev *dev,
                                    int where, unsigned int *val)
{
	u32 res;

	if (where & 3)
		return PCIBIOS_BAD_REGISTER_NUMBER;
	mkaddr(dev, where);
	res = *(volatile u32 *)VR41XX_PCICONFDREG;
	res = le32_to_cpu(res);
	*val = res;

	return PCIBIOS_SUCCESSFUL;
}

static int pcimt_write_config_byte (struct pci_dev *dev,
                                    int where, unsigned char val)
{
	mkaddr(dev, where);
	*(volatile u8 *)(VR41XX_PCICONFDREG + (where & 3)) = val;

	return PCIBIOS_SUCCESSFUL;
}

static int pcimt_write_config_word (struct pci_dev *dev,
                                    int where, unsigned short val)
{
	if (where & 1)
		return PCIBIOS_BAD_REGISTER_NUMBER;
	mkaddr(dev, where);
	*(volatile u16 *)(VR41XX_PCICONFDREG + (where & 3)) = le16_to_cpu(val);

	return PCIBIOS_SUCCESSFUL;
}

static int pcimt_write_config_dword (struct pci_dev *dev,
                                     int where, unsigned int val)
{
	if (where & 3)
		return PCIBIOS_BAD_REGISTER_NUMBER;
	mkaddr(dev, where);
	*(volatile u32 *)VR41XX_PCICONFDREG = le32_to_cpu(val);

	return PCIBIOS_SUCCESSFUL;
}

struct pci_ops vr4122_pci_ops = {
        pcimt_read_config_byte,
        pcimt_read_config_word,
        pcimt_read_config_dword,
        pcimt_write_config_byte,
        pcimt_write_config_word,
        pcimt_write_config_dword
};

void __init pcibios_update_irq(struct pci_dev *dev, int irq)
{
    /* assume it's a generic PCI interrupt */
    dev->irq = VR41XX_IRQ_PCI;
    /* if it's wired differently, correct the assumption */
    switch (dev->vendor) {
    case PCI_VENDOR_ID_NEC:
        switch (dev->device) {
        case PCI_DEVICE_ID_NEC_VRC4173:
        case PCI_DEVICE_ID_NEC_VRC4173_AC97:
        case PCI_DEVICE_ID_NEC_NAPCCARD:
        case PCI_DEVICE_ID_NEC_VRC4173_USB:     /* all three share the same IRQ */
            dev->irq = VR41XX_IRQ_4173;
            break;
        }
        break;

    case  PCI_VENDOR_ID_MEDIAQ:
        if (dev->device == PCI_DEVICE_ID_MEDIAQ_MQ200)
            dev->irq = VR41XX_IRQ_MQ200;
        break;
    }
}

static void __init pcibios_fixup_irqs(void)
{
    struct pci_dev *dev;
    int slot_num;
    int cnt = 0;

    pci_for_each_dev(dev) {
        slot_num = PCI_SLOT(dev->devfn);
	pcibios_update_irq(dev, cnt++);
    }
}

void __init pcibios_fixup_bus(struct pci_bus *b)
{
}

void pcibios_update_resource(struct pci_dev *dev, struct resource *root,
                        struct resource *res, int resource)
{
}

#define PCI_MEMBASE     0x10000000
#define PCI_MEMSIZE     0x05E00000
#define PCI_IOBASE      0x16000000
#define PCI_IOSIZE      0x02000000
#define PCI_IOEXCL      0x00010000      // Exclude first 64KB
#define PCI_MAW1        0x100F9010
#define PCI_MAW2        0x140FD014
#define PCI_IOAW        0x160FD000

// struct resource pci_mem =
// { "VR4122 PCI MEM", PCI_MEMBASE, PCI_MEMBASE+PCI_MEMSIZE-1 };
// struct resource pci_io  =
// { "VR4122 PCI I/O", PCI_IOBASE, PCI_IOBASE+PCI_IOSIZE-1 };

void __init pcibios_init(void)
{
        struct pci_ops *ops = &vr4122_pci_ops;

        pci_scan_bus(0, ops, NULL);
	pcibios_fixup_irqs();

//         request_resource(&iomem_resource, &pci_mem);
//         request_resource(&iomem_resource, &pci_io);

 	NEC_HARRIER_DISPLAY_LEDS(0xC001);
}

int __init pcibios_enable_device(struct pci_dev *dev)
{
        /* Not needed, since we enable all devices at startup.  */
        return 0;
}

void __init pcibios_align_resource(void *data, struct resource *res, unsigned long size)
{
}

char * __init pcibios_setup(char *str)
{
        /* Nothing to do for now.  */

        return str;
}

struct pci_fixup pcibios_fixups[] = {
        { 0 }
};

void *pci_alloc_consistent(struct pci_dev *hwdev, size_t size,
                           dma_addr_t *dma_handle)
{
        void *ret;
        int gfp = GFP_ATOMIC;
        int order = get_order(size);

        if (hwdev == NULL || hwdev->dma_mask != 0xffffffff)
                gfp |= GFP_DMA;
        ret = (void *)__get_free_pages(gfp, order);

        if (ret != NULL) {
                memset(ret, 0, size);
                *dma_handle = virt_to_bus(ret);
        }
        dma_cache_wback_inv((int) ret, PAGE_SIZE << order);
        return KSEG1ADDR(ret);
}

void pci_free_consistent(struct pci_dev *hwdev, size_t size,
                         void *vaddr, dma_addr_t dma_handle)
{
        free_pages((unsigned long)vaddr, get_order(size));
}

#endif /* CONFIG_PCI */
