/*
 * linux/arch/mips/vr41xx/vrc4173-ps2kbd.c
 *
 * Platform support for NEC Harrier.
 *
 * Copyright (C) 2000 Michael R. McDonald
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * The PS2 code originally came from drivers/char/kc_kbd.c
 *
 */

#include <linux/module.h>
#include <linux/sched.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/pc_keyb.h>
#include <linux/kbd_kern.h>
#include <linux/kbd_ll.h>
#include <asm/io.h>
#include <asm/keyboard.h>
#include <asm/vr41xx.h>
#include <asm/vrc4173.h>

#include <asm/vr41xx-platdep.h>

extern struct pci_dev *vrc4173_dev;

spinlock_t vrc4173_controller_lock = SPIN_LOCK_UNLOCKED;

int vrc4173_PS2_wait_for_input(struct pci_dev *pdev, int channel);
void vrc4173_PS2_write_data(struct pci_dev *pdev, int channel, unsigned short data);

char *vrc4173_PS2_init_kbd(struct pci_dev *pdev, int channel)
{
    long ioaddr;
    unsigned short status;

    ioaddr = pci_resource_start (pdev, 0);

    /* enable the port and turn off interrupts */
    outw(0x0, ioaddr + VRC4173_PS2CH1CTRL + channel);
    
//     outw(1, ioaddr + VRC4173_PS2CH1RST + channel);
//     mdelay(200);
//     status = PS2_wait_for_input(pdev, channel);
//     if ((status != 0) && (status != KBD_REPLY_POR)) {
// 	printk("VRC4173: keyboard reset status = 0x%x\n", status);
// 	return "Keyboard failed reset";
//     }

	/*
	 * Test the keyboard interface.
	 * This seems to be the only way to get it going.
	 * If the test is successful a x55 is placed in the input buffer.
	 */
	vrc4173_PS2_write_data(pdev, channel, KBD_CCMD_SELF_TEST);
	if (vrc4173_PS2_wait_for_input(pdev, channel) != KBD_REPLY_ACK)
	    return "Keyboard failed self test";

	/*
	 * Perform a keyboard interface test.  This causes the controller
	 * to test the keyboard clock and data lines.  The results of the
	 * test are placed in the input buffer.
	 */
	vrc4173_PS2_write_data(pdev, channel, KBD_CCMD_KBD_TEST);
	if (vrc4173_PS2_wait_for_input(pdev, channel) != KBD_REPLY_ACK)
	    return "Keyboard interface failed self test";

    /*
     * Enable the keyboard by allowing the keyboard clock to run.
     */
    vrc4173_PS2_write_data(pdev, channel, KBD_CCMD_KBD_ENABLE);

    /*
     * Reset keyboard. If the read times out
     * then the assumption is that no keyboard is
     * plugged into the machine.
     * This defaults the keyboard to scan-code set 2.
     *
     * Set up to try again if the keyboard asks for RESEND.
     */
    do {
	vrc4173_PS2_write_data(pdev, channel, KBD_CMD_RESET);
	status = vrc4173_PS2_wait_for_input(pdev, channel);
	if (status == KBD_REPLY_ACK)
	    break;
	if (status != KBD_REPLY_RESEND)
	    return "Keyboard reset failed, no ACK";
    } while (1);

    if (vrc4173_PS2_wait_for_input(pdev, channel) != KBD_REPLY_POR)
	return "Keyboard reset failed, no POR";

    /*
     * Set keyboard controller mode. During this, the keyboard should be
     * in the disabled state.
     *
     * Set up to try again if the keyboard asks for RESEND.
     */
    do {
	vrc4173_PS2_write_data(pdev, channel, KBD_CMD_DISABLE);
	status = vrc4173_PS2_wait_for_input(pdev, channel);
	if (status == KBD_REPLY_ACK)
	    break;
	if (status != KBD_REPLY_RESEND)
	    return "Disable keyboard: no ACK";
    } while (1);

    vrc4173_PS2_write_data(pdev, channel, KBD_CCMD_WRITE_MODE);
    vrc4173_PS2_write_data(pdev, channel, KBD_MODE_KBD_INT
		   | KBD_MODE_SYS
		   | KBD_MODE_DISABLE_MOUSE
		   | KBD_MODE_KCC);

    /* ibm powerpc portables need this to use scan-code set 1 -- Cort */
    vrc4173_PS2_write_data(pdev, channel, KBD_CCMD_READ_MODE);
    if (!(vrc4173_PS2_wait_for_input(pdev, channel) & KBD_MODE_KCC)) {
	/*
	 * If the controller does not support conversion,
	 * Set the keyboard to scan-code set 1.
	 */
	vrc4173_PS2_write_data(pdev, channel, 0xF0);
	vrc4173_PS2_wait_for_input(pdev, channel);
	vrc4173_PS2_write_data(pdev, channel, 0x01);
	vrc4173_PS2_wait_for_input(pdev, channel);
    }
	
    vrc4173_PS2_write_data(pdev, channel, KBD_CMD_ENABLE);
    if (vrc4173_PS2_wait_for_input(pdev, channel) != KBD_REPLY_ACK)
	return "Enable keyboard: no ACK";

    /*
     * Finally, set the typematic rate to maximum.
     */
    vrc4173_PS2_write_data(pdev, channel, KBD_CMD_SET_RATE);
    if (vrc4173_PS2_wait_for_input(pdev, channel) != KBD_REPLY_ACK)
	return "Set rate: no ACK";
    vrc4173_PS2_write_data(pdev, channel, 0x00);
    if (vrc4173_PS2_wait_for_input(pdev, channel) != KBD_REPLY_ACK)
	return "Set rate: no ACK (0x00)";

    outw(0x18, ioaddr + VRC4173_PS2CH1CTRL + channel);
    return NULL; /* success */
}

void vrc4173_handle_ps2_kbd(struct pci_dev *pdev, int index)
{
    long ioaddr;
    unsigned short status, data;
    int down = 1;
    int channel = 0x0;

    ioaddr = pci_resource_start (pdev, 0);
    
    while ((status = inw(ioaddr + VRC4173_PS2CH1CTRL + channel)) & 0x1) {
	data = inw(ioaddr + VRC4173_PS2CH1DATA + channel);
	NEC_HARRIER_DISPLAY_LEDS(data);
	printk("keyboard 0x%04x\n", data);
	if (data == 0xf0)
	    down = 0;
	else {
	    handle_scancode(data, down);
	    down = 1;
	}
    }
    tasklet_schedule(&keyboard_tasklet);
}

